/***************************************************************************
 *   SETUP.C  - Setup utility for TEKWAR & WITCHAVEN game                  *
 *                                                                         *
 *   use "WCL SETUP.C" to compile (no need for 32 bit)                     *
 *                                                                         *
 *                                                     06/25/95 Les Bird   *
 ***************************************************************************/

// EG TODO: WH2 has more keys on offer
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <setjmp.h>
#include <stdarg.h>
#include <graph.h>
#include <io.h>
#include <process.h>
#include "keycodes.h"
// EG 23 Aug 2017
#include "keydefs.h"

#define   FONTHEIGHT     8
#define   FONTWIDTH      8

#define   NUMOPTS        12

// EG in KEYDEFS.H
//#define   NUMKEYS        27
//#define   MOREOPTS       7         // mouse buttons, joystick on/off/buttons
#define   MOREOPTS       8         // mouse buttons, joystick on/off/buttons
                                   // +1 for mouse 3rd button

void eg_parse_config(void);
int eg_writeprefs(void);

char option[NUMOPTS+NUMKEYS+MOREOPTS];

// EG 23 Aug 2017 - We want to break up option[] eventually
char keyopt[NUMKEYS];

// EG 23 Aug 2017: Other values that we do not want setup to clobber
//                 even if they might not be used here
//                 Defaults are set, for a full CFG "out of the box"
int mousxspeed = 4;
int mousyspeed = 4;
int brightness = 0;
int digilevel = 10;
int musiclevel = 10;
int goreon = 1;
int difficulty = 2;
int eg_mlooklock = 1;
int eg_mlookinvert = 0;
// 15 Oct 2015
int runlock = 0;

volatile
char keyc;

int  curx,cury,
     menupos,
     mouse,
     mouseon,
     mx,oldmx,
     my,oldmy,
     mb,oldmb,
     selopt;

struct videoconfig vc;
struct textsettings ts;

char textbuf[160];

// EG 18 Oct 2017: Fix defaults to match KEYDEFS.H order and WH2 reference card
char defkeys[]={
     0xC8,
	 0xD0,
	 0xCB,
	 0xCD,
	 0x2A,
	 0x38,
	 0x1d,
	 0x39,
     0x1e,
	 0x2c,
	 0x49,
	 0x51,
	 0x47,
	 0x33,
	 0x34,
     0x0F,
	 0x0D,
	 0x0C,
	 0x1C,
	 0x29,
	 0xFF,
	 0xFF,
     0x4F,
	 0x52,
	 0x53,
	 0xFF,
	 0xFF,
	 0x1B,
	 0x1A,
	 0x1F
};


#define   MAINMENU            0
#define   CONTROLMENU         1
#define   SOUNDMENU           2
#define   VIDEOMENU           3
#define   EXITMENU            4
#define   MOUSEMENU           5
#define   JSTICKMENU          6
#define   ACTIONMENU          7

// Divide the menu array into sections
#define   MAINMENUOPTIONS     4
#define   MAINMENUSTART       0
#define   MAINMENUEND         (MAINMENUSTART+MAINMENUOPTIONS)
#define   CONTROLMENUOPTIONS  32
#define   CONTROLMENUSTART    MAINMENUEND
#define   CONTROLMENUEND      (CONTROLMENUSTART+CONTROLMENUOPTIONS)
#define   SOUNDMENUOPTIONS    4
#define   SOUNDMENUSTART      CONTROLMENUEND
#define   SOUNDMENUEND        (SOUNDMENUSTART+SOUNDMENUOPTIONS)
#define   MUSICMENUOPTIONS    0
#define   MUSICMENUSTART      SOUNDMENUEND
#define   MUSICMENUEND        (MUSICMENUSTART+MUSICMENUOPTIONS)
#define   VIDEOMENUOPTIONS    2
#define   VIDEOMENUSTART      MUSICMENUEND
#define   VIDEOMENUEND        (VIDEOMENUSTART+VIDEOMENUOPTIONS)
#define   EXITMENUOPTIONS     3
#define   EXITMENUSTART       VIDEOMENUEND
#define   EXITMENUEND         (EXITMENUSTART+EXITMENUOPTIONS)
// EG Was 4, +2 for new mouselook options
//           +1 for mouse 3rd button (31 Aug 2017)
//           +1 for autorun (16 Oct 2017)
#define   MOUSEMENUOPTIONS    8
#define   MOUSEMENUSTART      EXITMENUEND
#define   MOUSEMENUEND        (MOUSEMENUSTART+MOUSEMENUOPTIONS)
#define   JSTICKMENUOPTIONS   6
#define   JSTICKMENUSTART     MOUSEMENUEND
#define   JSTICKMENUEND       (JSTICKMENUSTART+JSTICKMENUOPTIONS)
#define   ACTIONMENUOPTIONS   6
#define   ACTIONMENUSTART     JSTICKMENUEND
#define   ACTIONMENUEND       (ACTIONMENUSTART+ACTIONMENUOPTIONS)

#define   MAINMENUY      24
#define   SOUNDMENUY     142
#define   CONTROLMENUY   142
#define   VIDEOMENUY     200
#define   EXITMENUY      142
#define   MOUSEMENUY     142
#define   JSTICKMENUY    142
#define   ACTIONMENUY    126

struct menu {
     int  x;
     int  y;
     int  fore;
     int  back;
     char *label;
} menu[]={
     { -1,MAINMENUY+00    ,14, 1,"CONTROL CONFIGURATION"},
//     { -1,MAINMENUY+16    ,14, 1,"MUSIC/SOUND CONFIGURATION"},
     { -1,MAINMENUY+16    ,14, 1,NULL},
     { -1,MAINMENUY+32    ,14, 1,"VIDEO MODE CONFIGURATION"},
     { -1,MAINMENUY+48    ,12, 1,"EXIT SETUP UTILITY"},
     { 48,CONTROLMENUY-14 ,10, 8,"CONFIG MOUSE + RUN "},
     { 48,CONTROLMENUY+00 ,10, 8,"CONFIGURE  JOYSTICK"},
     { 48,CONTROLMENUY+14 ,10, 8,"MOVE FORWARD......."},
     { 48,CONTROLMENUY+28 ,10, 8,"MOVE BACKWARD......"},
     { 48,CONTROLMENUY+42 ,10, 8,"TURN LEFT.........."},
     { 48,CONTROLMENUY+56 ,10, 8,"TURN RIGHT........."},
     { 48,CONTROLMENUY+70 ,10, 8,"RUN................"},
     { 48,CONTROLMENUY+84 ,10, 8,"STRAFE MODE........"},
     { 48,CONTROLMENUY+98 ,10, 8,"USE WEAPON........."},
     { 48,CONTROLMENUY+112,10, 8,"OPEN/CLOSE........."},
     { 48,CONTROLMENUY+126,10, 8,"JUMP..............."},
     { 48,CONTROLMENUY+140,10, 8,"CROUCH............."},
     { 48,CONTROLMENUY+154,10, 8,"LOOK UP............"},
     { 48,CONTROLMENUY+168,10, 8,"LOOK DOWN.........."},
     {344,CONTROLMENUY+00 ,10, 8,"CENTER VIEW........"},
     {344,CONTROLMENUY+14 ,10, 8,"STRAFE LEFT........"},
     {344,CONTROLMENUY+28 ,10, 8,"STRAFE RIGHT......."},
     {344,CONTROLMENUY+42 ,10, 8,"MAP MODE..........."},
//     {344,CONTROLMENUY+56 ,10, 8,NULL},
     {344,CONTROLMENUY+56 ,10, 8,"MAP MODE ZOOM IN..."},
     {344,CONTROLMENUY+70 ,10, 8,"MAP MODE ZOOM OUT.."},
//     {344,CONTROLMENUY+84 ,10, 8,NULL},
     {344,CONTROLMENUY+84 ,10, 8,"USE POTION........."},
     {344,CONTROLMENUY+98 ,10, 8,"CAST SPELL........."},
     {344,CONTROLMENUY+112,10, 8,NULL},
     {344,CONTROLMENUY+112,10, 8,NULL},
     {344,CONTROLMENUY+126,10, 8,NULL},
     {344,CONTROLMENUY+112,10, 8,"FLY UP............."},
     {344,CONTROLMENUY+126,10, 8,"FLY DOWN..........."},
     {344,CONTROLMENUY+112,10, 8,NULL},
     {344,CONTROLMENUY+112,10, 8,NULL},
     {344,CONTROLMENUY+140,10, 8,"NEXT POTION........"},
     {344,CONTROLMENUY+154,10, 8,"PREVIOUS POTION...."},  
     {344,CONTROLMENUY+168,10, 8,"USE/STOW SHIELD...."},
     { -1,SOUNDMENUY+00   ,10, 8,"MUSIC........... ON"},
     { -1,SOUNDMENUY+16   ,10, 8,"SOUND EFFECTS... ON"},
     { -1,SOUNDMENUY+32   ,10, 8,"EXECUTE SOUND CONFIGURATION PROGRAM"},
     { -1,SOUNDMENUY+48   ,10, 8,"RETURN TO MAIN MENU"},
     {232,VIDEOMENUY+00   ,10, 8,"320 x 200 - 256 colors"},
     {232,VIDEOMENUY+48   ,10, 8,"640 x 480 - 256 colors"},
     { -1,EXITMENUY+00    ,10, 8,"EXIT AND SAVE"},
     { -1,EXITMENUY+16    ,10, 8,"EXIT WITHOUT SAVING"},
     { -1,EXITMENUY+32    ,10, 8,"CANCEL AND RETURN TO MENU"},
     { 48,MOUSEMENUY+00   ,10, 8,"MOUSE ENABLED......"},
     { 48,MOUSEMENUY+16   ,10, 8,"LEFT BUTTON."},
     { 48,MOUSEMENUY+32   ,10, 8,"RIGHT BUTTON"},
// EG
     { 48,MOUSEMENUY+48   ,10, 8,"MID BUTTON.."},
     { 48,MOUSEMENUY+64   ,10, 8,"ALWAYS MOUSE LOOK"},
     { 48,MOUSEMENUY+80   ,10, 8,"INVERT MOUSE LOOK"},
     { 48,MOUSEMENUY+96   ,10, 8,"ALWAYS RUN"},
     { 48,MOUSEMENUY+128   ,10, 8,"BACK TO CONTROL MENU"},
     { 48,JSTICKMENUY+00  ,10, 8,"JOYSTICK ENABLED..."},
     { 48,JSTICKMENUY+16  ,10, 8,"BUTTON 1.."},
     { 48,JSTICKMENUY+32  ,10, 8,"BUTTON 2.."},
     { 48,JSTICKMENUY+48  ,10, 8,"BUTTON 3.."},
     { 48,JSTICKMENUY+64  ,10, 8,"BUTTON 4.."},
     { 48,JSTICKMENUY+96  ,10, 8,"BACK TO CONTROL MENU"},
     
     {344,ACTIONMENUY+10  ,10, 8,"RUN MODE"},
     {344,ACTIONMENUY+30  ,10, 8,"STRAFE MODE"},
     {344,ACTIONMENUY+50  ,10, 8,"USE WEAPON"},
     {344,ACTIONMENUY+70  ,10, 8,"OPEN/CLOSE"},
     
     {344,ACTIONMENUY+90  ,10, 8,"JUMP"},
     {344,ACTIONMENUY+110  ,10, 8,"CROUCH"},

     
     {344,ACTIONMENUY+60  ,10, 8,NULL},
     {344,ACTIONMENUY+70  ,10, 8,NULL},
     {344,ACTIONMENUY+80  ,10, 8,NULL},
     {344,ACTIONMENUY+90  ,10, 8,NULL},
     {344,ACTIONMENUY+100 ,10, 8,NULL},
     {344,ACTIONMENUY+110 ,10, 8,NULL},
     {344,ACTIONMENUY+120 ,10, 8,NULL},
     {344,ACTIONMENUY+130 ,10, 8,NULL},
     {344,ACTIONMENUY+140 ,10, 8,NULL},
     {344,ACTIONMENUY+150 ,10, 8,NULL},
     {344,ACTIONMENUY+160 ,10, 8,NULL},
     {344,ACTIONMENUY+170 ,10, 8,NULL},
     {344,ACTIONMENUY+180 ,10, 8,NULL}
     
     #if 0
     {344,ACTIONMENUY+40  ,10, 8,"JUMP"},
     {344,ACTIONMENUY+50  ,10, 8,"CROUCH"},
     {344,ACTIONMENUY+60  ,10, 8,"LOOK UP"},
     {344,ACTIONMENUY+70  ,10, 8,"LOOK DOWN"},
     {344,ACTIONMENUY+80  ,10, 8,"CENTER VIEW"},
     {344,ACTIONMENUY+90  ,10, 8,"STRAFE LEFT"},
     {344,ACTIONMENUY+100 ,10, 8,"STRAFE RIGHT"},
     {344,ACTIONMENUY+110 ,10, 8,"VIEW 2D MAP"},
     {344,ACTIONMENUY+120 ,10, 8,"ZOOM IN"},
     {344,ACTIONMENUY+130 ,10, 8,"ZOOM OUT"},
     {344,ACTIONMENUY+140 ,10, 8,"USE POTION"},
     {344,ACTIONMENUY+150 ,10, 8,"CAST SPELL"},
     {344,ACTIONMENUY+160 ,10, 8,"LOOK UP/DOWN MODE"},
     {344,ACTIONMENUY+170 ,10, 8,"FLY UP"},
     {344,ACTIONMENUY+180 ,10, 8,"FLY DOWN"}
     #endif
};

int  menustarts[]={
     MAINMENUSTART,CONTROLMENUSTART,SOUNDMENUSTART,VIDEOMENUSTART,
     EXITMENUSTART,MOUSEMENUSTART,JSTICKMENUSTART,ACTIONMENUSTART
},menuends[]={
     MAINMENUEND,CONTROLMENUEND,SOUNDMENUEND,VIDEOMENUEND,
     EXITMENUEND,MOUSEMENUEND,JSTICKMENUEND,ACTIONMENUEND
};


int  musicon=1,
     soundon=1;

char *musiconstr={"MUSIC........... ON"},
     *musicoffstr={"MUSIC........... OFF"},
     *soundonstr={"SOUND EFFECTS... ON"},
     *soundoffstr={"SOUND EFFECTS... OFF"};


void (far *oldkeyint)(void);

int
getBIOSvmode(void)
{
     union REGPACK regs;

     memset(&regs,0,sizeof(union REGPACK));
     regs.h.ah=0x0F;
     regs.h.al=0x00;
     intr(0x10,&regs);
     return(regs.h.al);
}

void
setBIOSvmode(int m)
{
     union REGPACK regs;

     memset(&regs,0,sizeof(union REGPACK));
     regs.h.ah=0x00;
     regs.h.al=m;
     intr(0x10,&regs);
}

void interrupt far
newkeyint(void)
{
     if (inp(0x64)&0x01) {         /* read the status reg, output buf full?*/
          keyc=inp(0x60);          /* read the scan code..                 */
          outp(0x20,0x20);         /* send EOI code (end of interrupt)     */
     }
     _enable();                    /* turn on interrupt processing.             */
}

char
getkey(void)
{
     keyc=0;
     while (keyc == 0);
     return(keyc);
}

char
newgetch(void)
{
     int  extended=0;

     oldkeyint=_dos_getvect(0x09);
     _dos_setvect(0x09,newkeyint);
more:
     getkey();
     if (keyc == 224) {
          extended=1;
          getkey();
          if (keyc == 42 || keyc == 170) {
               goto more;
          }
     }
     else if (keyc > 128) {
          goto more;
     }
     _dos_setvect(0x09,oldkeyint);
     return(keyc+(extended<<7));
}

int
resetmouse(void)
{
     union REGS regs;

     regs.x.ax=0x0000;
     int86(0x33,&regs,&regs);
     if (regs.x.ax != 0) {
          regs.x.ax=0x0021;
          int86(0x33,&regs,&regs);
          return(1);
     }
     return(0);
}

void
showmouse(void)
{
     union REGS regs;

     if (mouse) {
          if (!mouseon) {
               regs.x.ax=0x0001;
               int86(0x33,&regs,&regs);
               mouseon=1;
          }
     }
}

void
hidemouse(void)
{
     union REGS regs;

     if (mouse) {
          if (mouseon) {
               regs.x.ax=0x0002;
               int86(0x33,&regs,&regs);
               mouseon=0;
          }
     }
}

void
getmouse(int *x,int *y,int *b)
{
     union REGS regs;

     if (mouse) {
          regs.x.ax=0x0003;
          int86(0x33,&regs,&regs);
          *x=regs.x.cx;
          *y=regs.x.dx;
          *b=regs.x.bx;
     }
}

void
gprintf(short x,short y,short color,char *stg,...)
{
     va_list valist;

     va_start(valist,stg);
     vsprintf(textbuf,stg,valist);
     va_end(valist);
     if (x == -1) {
          x=(vc.numxpixels>>1)-(strlen(textbuf)<<2);
     }
     if (y == -1) {
          y=(vc.numypixels>>1);
     }
     _setcolor(0);
     _moveto(x,y);
     _outgtext(textbuf);
     if (color > 0) {
          _setcolor(color);
          _moveto(x+1,y+1);
          _outgtext(textbuf);
     }
     curx=x;
     cury=y;
}

void
drawheader(void)
{
     _setcolor(7);
     _rectangle(_GFILLINTERIOR,1,1,vc.numxpixels-2,16);
     _setcolor(15);
     _rectangle(_GBORDER,0,0,vc.numxpixels-1,16);
     gprintf(-1,4,0,"EGWHAVEN SETUP UTILITY v1.1"); // EG 23 Aug 2017: Change name
}

int
centerx(int menopt)
{
     return((vc.numxpixels>>1)-((strlen(menu[menopt].label)*FONTWIDTH)>>1));
}

void
menuopthilite(int opt,int onoff)
{
     int  x1,y1,x2,y2;

     if (menu[opt].label != NULL) {
          x1=menu[opt].x;
          if (x1 == -1) {
               x1=centerx(opt);
          }
          x2=x1+(strlen(menu[opt].label)*FONTWIDTH);
          y1=menu[opt].y;
          y2=y1+FONTHEIGHT;
          if (onoff) {
               _setcolor(15);
          }
          else {
               _setcolor(menu[opt].back);
          }
          if (opt >= ACTIONMENUSTART && opt < ACTIONMENUEND) {
               _rectangle(_GBORDER,x1-3,y1-2,x2+3,y2+2);
               _rectangle(_GBORDER,x1-2,y1-1,x2+2,y2+1);
          }
          else {
               _rectangle(_GBORDER,x1-3,y1-3,x2+3,y2+3);
               _rectangle(_GBORDER,x1-2,y1-2,x2+2,y2+2);
          }
     }
}

void
drawmenuopt(int oldopt,int newopt)
{
     menuopthilite(oldopt,0);
     menuopthilite(newopt,1);
}

void
drawmainmenu(void)
{
     int  i;

     for (i=MAINMENUSTART ; i < MAINMENUEND ; i++) {
          if (menu[i].label != NULL) {
               gprintf(menu[i].x,menu[i].y,menu[i].fore,menu[i].label);
          }
     }
}

// Initial draw of mouse options
void
showmouseline(int i)
{
     int  a;
    switch (i)
    {
     case MOUSEMENUSTART:
          _setcolor(menu[i].back);
          _rectangle(_GFILLINTERIOR,216,menu[i].y,216+48,
               menu[i].y+FONTHEIGHT);
          gprintf(menu[i].x,menu[i].y,menu[i].fore,
               menu[i].label);
          gprintf(216,menu[i].y,14,option[3] == 0 ? "NO" : "YES");
          break;
     case MOUSEMENUSTART+1:
     case MOUSEMENUSTART+2:
// EG 31 Aug 2017
     case MOUSEMENUSTART+3:
          if (i == MOUSEMENUSTART+3)
            a=NUMOPTS+NUMKEYS+7;//was 1
          else
            a=NUMOPTS+NUMKEYS+(i-MOUSEMENUSTART)-1;//was 1
          a=option[a]+ACTIONMENUSTART-4;
          gprintf(216,menu[i].y,14,menu[a].label);
          break;
     case MOUSEMENUSTART+4:
          _setcolor(menu[i].back);
          _rectangle(_GFILLINTERIOR,216,menu[i].y,216+48,
               menu[i].y+FONTHEIGHT);
          gprintf(menu[i].x,menu[i].y,menu[i].fore,
               menu[i].label);
          gprintf(216,menu[i].y,14,eg_mlooklock == 0 ? "NO" : "YES");
          break;
     case MOUSEMENUSTART+5:
          _setcolor(menu[i].back);
          _rectangle(_GFILLINTERIOR,216,menu[i].y,216+48,
               menu[i].y+FONTHEIGHT);
          gprintf(menu[i].x,menu[i].y,menu[i].fore,
               menu[i].label);
          gprintf(216,menu[i].y,14,eg_mlookinvert == 0 ? "NO" : "YES");
          break;
     case MOUSEMENUSTART+6:
          _setcolor(menu[i].back);
          _rectangle(_GFILLINTERIOR,216,menu[i].y,216+48,
               menu[i].y+FONTHEIGHT);
          gprintf(menu[i].x,menu[i].y,menu[i].fore,
               menu[i].label);
          gprintf(216,menu[i].y,14,runlock == 0 ? "NO" : "YES");
          break;		  
    }
}

void
showmousesettings(void)
{
     int  i;

     for (i=MOUSEMENUSTART ; i < MOUSEMENUEND ; i++) {
          if (menu[i].label != NULL) {
               gprintf(menu[i].x,menu[i].y,menu[i].fore,menu[i].label);
               if (i < MOUSEMENUEND-1) {
                    showmouseline(i);
               }
          }
     }
}

void
showjstickline(int i)
{
     int  a;

     if (i == JSTICKMENUSTART) {
          _setcolor(menu[i].back);
          _rectangle(_GFILLINTERIOR,216,menu[i].y,216+48,menu[i].y+FONTHEIGHT);
          gprintf(menu[i].x,menu[i].y,menu[i].fore,menu[i].label);
          gprintf(216,menu[i].y,14,
               option[NUMOPTS+NUMKEYS+2] == 0 ? "NO" : "YES");
     }
     else {
          a=NUMOPTS+NUMKEYS+3+(i-JSTICKMENUSTART)-1;
          a=option[a]+ACTIONMENUSTART-4;
          gprintf(216,menu[i].y,14,menu[a].label);
     }
}

void
showjsticksettings(void)
{
     int  i;

     for (i=JSTICKMENUSTART ; i < JSTICKMENUEND ; i++) {
          if (menu[i].label != NULL) {
               gprintf(menu[i].x,menu[i].y,menu[i].fore,menu[i].label);
               if (i < JSTICKMENUEND-1) {
                    showjstickline(i);
               }
          }
     }
}

// Initial display of control settings
void
showcontrolline(int i)
{
     char c;

     c=option[i-CONTROLMENUSTART+NUMOPTS-2];
     // EG TODO: Make this use keyopt directly if possible
     //c=keyopt[i-CONTROLMENUSTART];
     if (strcmp(keys[c&0x7F],"CTRL") == 0
        || strcmp(keys[c&0x7F],"ALT") == 0) {
          gprintf(menu[i].x+168,menu[i].y,14,"%s%s",
               (c&0x80) != 0 ? "R" : "L",keys[c&0x7F]);
     }
     else if (strcmp(keys[c&0x7F],"RETURN") == 0) {
          gprintf(menu[i].x+168,menu[i].y,14,"%s",
               (c&0x80) != 0 ? "KP ENTER" : "ENTER");
     }
     else {
          gprintf(menu[i].x+168,menu[i].y,14,keys[c&0x7F]);
     }
}

void
showcontrolsettings(void)
{
     int  i,n;
     char c;

     _setcolor(0);
     _moveto(vc.numxpixels>>1,116);
     _lineto(vc.numxpixels>>1,vc.numypixels-25);
     for (i=CONTROLMENUSTART ; i < CONTROLMENUEND ; i++) {
          if (menu[i].label != NULL) {
               gprintf(menu[i].x,menu[i].y,menu[i].fore,menu[i].label);
               if (i >= CONTROLMENUSTART+2) {
                    showcontrolline(i);
               }
          }
     }
}

void
docontrolaction(int domenu,int opt)
{
     int  x1,x2,y1,y2;

     menuopthilite(opt,0);
     x1=menu[opt].x+168;
     y1=menu[opt].y;
     x2=x1+(9*FONTWIDTH);
     y2=y1+FONTHEIGHT;
     _setcolor(0);
     _rectangle(_GFILLINTERIOR,x1,y1,x2,y2);
     option[opt-menustarts[domenu]+NUMOPTS-2]=newgetch();
     _setcolor(menu[opt].back);
     _rectangle(_GFILLINTERIOR,x1,y1,x2,y2);
     menuopthilite(opt,1);
     showcontrolline(opt);
}

void
showsoundsettings(void)
{
     int  i;

     for (i=SOUNDMENUSTART ; i < SOUNDMENUEND ; i++) {
          if (menu[i].label != NULL) {
               gprintf(menu[i].x,menu[i].y,menu[i].fore,menu[i].label);
          }
     }
}

void
showmusicsettings(void)
{
     int  i;

     for (i=MUSICMENUSTART ; i < MUSICMENUEND ; i++) {
          if (menu[i].label != NULL) {
               gprintf(menu[i].x,menu[i].y,menu[i].fore,menu[i].label);
          }
     }
}

void
showvideosettings(void)
{
     int  i,x1,x2,y1,y2;

     if (option[0] == 1 && option[6] == 0x62) {
          strcpy(textbuf,"VIDEO MODE........... 640 x 480 - 256 colors");
     }
     else if (option[0] == 0 && option[6] == 0x62) {
          strcpy(textbuf,"VIDEO MODE........... 360 x 360 - 256 colors");
     }
     else if (option[0] == 1 && option[6] == 0x61) {
          strcpy(textbuf,"VIDEO MODE........... 640 x 400 - 256 colors");
     }
     else {
          strcpy(textbuf,"VIDEO MODE........... 320 x 200 - 256 colors");
          option[0]=1;
          option[6]=0x60;
     }
     x1=(vc.numxpixels>>1)-((strlen(textbuf)*FONTWIDTH)>>1);
     x2=x1+(strlen(textbuf)*FONTWIDTH);
     y1=152;
     y2=y1+FONTHEIGHT;
     _setcolor(menu[VIDEOMENUSTART].back);
     _rectangle(_GFILLINTERIOR,x1,y1,x2,y2);
     gprintf(x1,y1,14,textbuf);
     for (i=VIDEOMENUSTART ; i < VIDEOMENUEND ; i++) {
          if (menu[i].label != NULL) {
               gprintf(menu[i].x,menu[i].y,menu[i].fore,menu[i].label);
          }
     }
}

void
showexitsettings(void)
{
     int  i;

     for (i=EXITMENUSTART ; i < EXITMENUEND ; i++) {
          if (menu[i].label != NULL) {
               gprintf(menu[i].x,menu[i].y,menu[i].fore,menu[i].label);
          }
     }
}

void
showactionsettings(void)
{
     int  i;

     for (i=ACTIONMENUSTART ; i < ACTIONMENUEND ; i++) {
          if (menu[i].label != NULL) {
               gprintf(menu[i].x,menu[i].y,menu[i].fore,menu[i].label);
          }
     }
}

void
drawoptsettings(int opt)
{
     _setcolor(8);
     if (opt+1 == ACTIONMENU) {
          _rectangle(_GFILLINTERIOR,vc.numxpixels>>1,117,
               vc.numxpixels-24,vc.numypixels-24);
     }
     else {
          _rectangle(_GFILLINTERIOR,24,100,vc.numxpixels-24,vc.numypixels-24);
     }
     _setcolor(15);
     _rectangle(_GBORDER,24,100,vc.numxpixels-24,vc.numypixels-24);
     if (opt+1 != ACTIONMENU) {
          gprintf(-1,104,15,"CURRENT SETTINGS");
          _setcolor(0);
          _moveto(25,116);
          _lineto(vc.numxpixels-25,116);
     }
     switch (opt+1) {
     case CONTROLMENU:
          showcontrolsettings();
          break;
     case SOUNDMENU:
          showsoundsettings();
          break;
     case VIDEOMENU:
          showvideosettings();
          break;
     case EXITMENU:
          showexitsettings();
          break;
     case MOUSEMENU:
          showmousesettings();
          break;
     case JSTICKMENU:
          showjsticksettings();
          break;
     case ACTIONMENU:
          showactionsettings();
          break;
     }
}

void
drawmain(int domenu)
{
     _setcolor(1);
     _rectangle(_GFILLINTERIOR,0,0,vc.numxpixels-1,vc.numypixels-1);
     _setcolor(15);
     _rectangle(_GBORDER,0,0,vc.numxpixels-1,vc.numypixels-1);
     drawheader();
     drawmainmenu();
     drawoptsettings(domenu);
     drawmenuopt(selopt,selopt);
}

void
runaudioexe(int domenu)
{
     hidemouse();
     _setvideomode(_TEXTC80);
     spawnl(P_WAIT,"setup.exe","setup.exe",NULL);
     _setvideomode(_ERESCOLOR);
     drawmain(domenu-1);
     showmouse();
}

void
main(void)
{
     int  domenu=0,done=0,i,len,mousemenu=0,omode,oldselopt,x,x1,x2,y1,y2;
     int  butopt,savdomenu,savselopt;
     char *crashstr=NULL;
//     char temp[10]={"ver1"};
     FILE *fp;
     void func1(void);
     
// EG: Ver1.dat is completely pointless, or?
#if 0
     if (access("ver1.dat",F_OK) != 0) {
        fp=fopen("ver1.dat","w");
        if( fp != NULL ) {
            fwrite(temp,sizeof(temp),1,fp);
            fclose(fp);
        }
     }
#endif

// EG: Don't try to use this file at all, since it is WH1 specific
#if 0
     if (access("setup.dat",F_OK) == 0) {
          fp=fopen("setup.dat","rb");
          fread(option,sizeof(char),NUMOPTS+NUMKEYS+MOREOPTS,fp);
          fclose(fp);
     }
     else {
#endif
          option[0]=1;
          option[1]=1;
          option[2]=1;
          option[3]=1;
          option[4]=0;
          option[5]=0;
          option[6]=0x60;
          option[7]=0x16;
          option[8]=0;
          option[9]=0;
          option[10]=0;
          option[11]=0;
          for (i=0 ; i < NUMKEYS ; i++) {
               option[i+NUMOPTS]=defkeys[i];
          }
	      // Establish a default here, just in case
          keys[KEYUNCAST] = 0xcf;
// EG: Bracket goes with above check that was removed
#if 0
     }
#endif
// EG: Read EGPREF.CFG
     if (access("egpref.cfg",F_OK) == 0)
     {
         eg_parse_config();
     }

     musicon=option[9];
     if (option[9] == 0) {
          menu[menustarts[SOUNDMENU]].label=musicoffstr;
     }
     else {
          menu[menustarts[SOUNDMENU]].label=musiconstr;
     }               
     soundon=option[11];
     if (option[11] == 0) {
          menu[menustarts[SOUNDMENU]+1].label=soundoffstr;
     }
     else {
          menu[menustarts[SOUNDMENU]+1].label=soundonstr;
     }

     omode=getBIOSvmode();
     _setvideomode(_ERESCOLOR);
     _getvideoconfig(&vc);
     _gettextsettings(&ts);
     drawheader();
     gprintf(0,20,7,"Initializing...");
     mouse=resetmouse();
     drawmain(domenu);
     showmouse();
     oldselopt=selopt=0;
     do {
          if (!kbhit()) {
               goto skipkeys;
          }
          if (mouseon) {
               hidemouse();
          }
          switch (getch()) {
          case 0:
               switch (getch()) {
               case 75:            // left arrow
                    break;
               case 77:            // right arrow
                    break;
               case 72:            // up arrow
                    do {
                         selopt--;
                         if (selopt < menustarts[domenu]) {
                              selopt=menuends[domenu]-1;
                         }
                    } while (menu[selopt].label == NULL);
                    drawmenuopt(oldselopt,selopt);
                    if (domenu == 0) {
                         drawoptsettings(selopt);
                    }
                    oldselopt=selopt;
                    break;
               case 80:            // down arrow
                    do {
                         selopt++;
                         if (selopt >= menuends[domenu]) {
                              selopt=menustarts[domenu];
                         }
                    } while (menu[selopt].label == NULL);
                    drawmenuopt(oldselopt,selopt);
                    if (domenu == 0) {
                         drawoptsettings(selopt);
                    }
                    oldselopt=selopt;
                    break;
               case 73:            // pgup
                    break;
               case 81:            // pgdn
                    break;
               case 68:            // F10
                    done=1;
                    break;
               default:
                    break;
               }
               break;
          case 0x20:               // spacebar
               break;
          case 0x0D:               // enter key
               switch (domenu) {
               case MAINMENU:      // main menu
                    if (selopt < MAINMENUOPTIONS) {
                         domenu=selopt+1;
                         selopt=menustarts[domenu];
                         drawmenuopt(oldselopt,selopt);
                         oldselopt=selopt;
                    }
                    break;
               case CONTROLMENU:   // control menu
                    if (selopt < menustarts[domenu]+2) {
                         _setcolor(8);
                         _rectangle(_GFILLINTERIOR,24,100,vc.numxpixels-24,
                              vc.numypixels-24);
                         _setcolor(15);
                         _rectangle(_GBORDER,24,100,vc.numxpixels-24,
                              vc.numypixels-24);
                         gprintf(-1,104,15,"CURRENT SETTINGS");
                         _setcolor(0);
                         _moveto(25,116);
                         _lineto(vc.numxpixels-25,116);
                         if (selopt == menustarts[domenu]) {
                              domenu=MOUSEMENU;
                         }
                         else {
                              domenu=JSTICKMENU;
                         }
                         drawoptsettings(domenu-1);
                         oldselopt=selopt=menustarts[domenu];
                         menuopthilite(selopt,1);
                    }
                    else {
                         docontrolaction(domenu,selopt);
                    }
                    break;
               case SOUNDMENU:     // sound menu
                    switch (selopt) {
                    case SOUNDMENUSTART:
                         _setcolor(8);
                         _rectangle(_GFILLINTERIOR,234,136,415,157);
                         if (musicon) {
                              menu[selopt].label=musicoffstr;
                              musicon=0;
                         }
                         else {
                              menu[selopt].label=musiconstr;
                              musicon=1;
                         }
                         option[9]=musicon;
                         showsoundsettings();
                         drawmenuopt(selopt,selopt);
                         break;
                    case SOUNDMENUSTART+1:
                         _setcolor(8);
                         _rectangle(_GFILLINTERIOR,234,152,415,172);
                         if (soundon) {
                              menu[selopt].label=soundoffstr;
                              soundon=0;
                         }
                         else {
                              menu[selopt].label=soundonstr;
                              soundon=1;
                         }
                         option[11]=soundon;
                         showsoundsettings();
                         drawmenuopt(selopt,selopt);
                         break;
                    case SOUNDMENUSTART+2:
                         runaudioexe(domenu);
                         break;
                    case SOUNDMENUSTART+3:
                         menuopthilite(selopt,0);
                         selopt=domenu-1;
                         menuopthilite(selopt,1);
                         oldselopt=selopt;
                         domenu=0;
                         break;
                    }
                    break;
//               case MUSICMENU:     // music menu
//                    break;
               case VIDEOMENU:     // video menu
                    switch (selopt) {
                    case VIDEOMENUSTART:
                         option[0]=1;
                         option[6]=0x60;
                         break;
                    case VIDEOMENUSTART+1:
                         option[0]=1;
                         option[6]=0x62;
                         break;
                    }
                    menuopthilite(selopt,0);
                    selopt=domenu-1;
                    menuopthilite(selopt,1);
                    oldselopt=selopt;
                    showvideosettings();
                    domenu=0;
                    break;
               case EXITMENU:
                    switch (selopt) {
                    case EXITMENUSTART:
                        if (eg_writeprefs()) // Because otherwise we have to copy the whole mess a bunch of times...
                        {
                              done=1;
                              atexit( func1 );
                        }
                        else {
                              done=2;
                        }
                        break;
                    case EXITMENUSTART+1:
                         done=1;
                         break;
                    default:
                         menuopthilite(selopt,0);
                         selopt=domenu-1;
                         menuopthilite(selopt,1);
                         oldselopt=selopt;
                         domenu=0;
                         break;
                    }
                    break;
               case MOUSEMENU:
                    switch (selopt) {
                    // Mouse on/off
                    case MOUSEMENUSTART:
                         option[3]^=1;
                         _setcolor(menu[selopt].back);
                         _rectangle(_GFILLINTERIOR,216,menu[selopt].y,
                              216+48,menu[selopt].y+FONTHEIGHT);
                         gprintf(menu[selopt].x,menu[selopt].y,
                              menu[selopt].fore,menu[selopt].label);
                         gprintf(216,menu[selopt].y,14,
                              option[3] == 0 ? "NO" : "YES");
                         break;
                    // Button 1
                    case MOUSEMENUSTART+1:
                    // Button 2
                    case MOUSEMENUSTART+2:
                      case MOUSEMENUSTART+3:
                           butopt=NUMOPTS+NUMKEYS;
                           if (selopt == MOUSEMENUSTART+2) {
                                butopt++;
                           }
                           else if (selopt == MOUSEMENUSTART+3) {
                               butopt += 7;
                           }
                         menuopthilite(selopt,0);
                         savdomenu=domenu;
                         domenu=ACTIONMENU;
                         savselopt=selopt;
                         oldselopt=selopt=menustarts[domenu];
                         drawoptsettings(domenu-1);
                         menuopthilite(selopt,1);
                         break;
                    // EG Always Mlook
                    case MOUSEMENUSTART+4:
                         eg_mlooklock^=1;
                         _setcolor(menu[selopt].back);
                         _rectangle(_GFILLINTERIOR,216,menu[selopt].y,
                              216+48,menu[selopt].y+FONTHEIGHT);
                         gprintf(menu[selopt].x,menu[selopt].y,
                              menu[selopt].fore,menu[selopt].label);
                         gprintf(216,menu[selopt].y,14,
                              eg_mlooklock == 0 ? "NO" : "YES");
                         break;
                    // EG Invert Mlook
                    case MOUSEMENUSTART+5:
                         eg_mlookinvert^=1;
                         _setcolor(menu[selopt].back);
                         _rectangle(_GFILLINTERIOR,216,menu[selopt].y,
                              216+48,menu[selopt].y+FONTHEIGHT);
                         gprintf(menu[selopt].x,menu[selopt].y,
                              menu[selopt].fore,menu[selopt].label);
                         gprintf(216,menu[selopt].y,14,
                              eg_mlookinvert == 0 ? "NO" : "YES");
                         break;
                    // EG Always run
                    case MOUSEMENUSTART+6:
                         runlock^=1;
                         _setcolor(menu[selopt].back);
                         _rectangle(_GFILLINTERIOR,216,menu[selopt].y,
                              216+48,menu[selopt].y+FONTHEIGHT);
                         gprintf(menu[selopt].x,menu[selopt].y,
                              menu[selopt].fore,menu[selopt].label);
                         gprintf(216,menu[selopt].y,14,
                              runlock == 0 ? "NO" : "YES");
                         break;                      
                    // Exit submenu
                    case MOUSEMENUSTART+7: // EG
                         domenu=CONTROLMENU;
                         oldselopt=selopt=menustarts[domenu];
                         drawoptsettings(domenu-1);
                         menuopthilite(selopt,1);
                         break;
                    }
                    break;
               case JSTICKMENU:
                    switch (selopt) {
                    case JSTICKMENUSTART:
                         option[NUMOPTS+NUMKEYS+2]^=1;
                         _setcolor(menu[selopt].back);
                         _rectangle(_GFILLINTERIOR,216,menu[selopt].y,
                              216+48,menu[selopt].y+FONTHEIGHT);
                         gprintf(menu[selopt].x,menu[selopt].y,
                              menu[selopt].fore,menu[selopt].label);
                         gprintf(216,menu[selopt].y,14,
                              option[NUMOPTS+NUMKEYS+2] == 0 ? "NO" : "YES");
                         break;
                    case JSTICKMENUSTART+1:
                    case JSTICKMENUSTART+2:
                    case JSTICKMENUSTART+3:
                    case JSTICKMENUSTART+4:
                         butopt=(NUMOPTS+NUMKEYS)+(selopt-(JSTICKMENUSTART+1))+3;
                         menuopthilite(selopt,0);
                         savdomenu=domenu;
                         domenu=ACTIONMENU;
                         savselopt=selopt;
                         oldselopt=selopt=menustarts[domenu];
                         drawoptsettings(domenu-1);
                         menuopthilite(selopt,1);
                         break;
                    case JSTICKMENUSTART+5:
                         domenu=CONTROLMENU;
                         oldselopt=selopt=menustarts[domenu];
                         drawoptsettings(domenu-1);
                         menuopthilite(selopt,1);
                         break;
                    }
                    break;
               case ACTIONMENU:
                    i=(selopt-ACTIONMENUSTART)+4;
                    option[butopt]=i;
                    _setcolor(8);
                    _rectangle(_GFILLINTERIOR,vc.numxpixels>>1,117,
                         vc.numxpixels-24,vc.numypixels-24);
                    domenu=savdomenu;
                    oldselopt=selopt=savselopt;
                    drawoptsettings(domenu-1);
                    menuopthilite(selopt,1);
                    break;
               }
               break;
          case 0x1B:               // ESC key
               if (domenu != 0) {
                    menuopthilite(selopt,0);
                    if (domenu == MOUSEMENU || domenu == JSTICKMENU
                       || domenu == ACTIONMENU) {
                         if (domenu == ACTIONMENU) {
                              domenu=savdomenu;
                         }
                         else {
                              domenu=CONTROLMENU;
                         }
                         selopt=menustarts[domenu];
                         drawoptsettings(domenu-1);
                    }
                    else {
                         selopt=domenu-1;
                         domenu=0;
                    }
                    menuopthilite(selopt,1);
                    oldselopt=selopt;
               }
               else {
                    done=1;
               }
               break;
          default:
               break;
          }
// It seems we need to duplicate things here as well
skipkeys:
          if (mouse) {
               getmouse(&mx,&my,&mb);
               if (mx != oldmx || my != oldmy || mb != oldmb) {
                    if (mouseon) {
                         hidemouse();
                    }
                    _setcolor(1);
                    _rectangle(_GFILLINTERIOR,1,vc.numypixels-12,
                         vc.numxpixels-1,vc.numypixels-1);
//                    gprintf(0,vc.numypixels-10,15,"mx=%d my=%d mb=%X",
//                         mx,my,mb);
                    oldmx=mx;
                    oldmy=my;
                    oldmb=mb;
                    if ((mb&1) != 0) {
                         switch (mousemenu) {
                         case MAINMENU:
                              if (selopt < menuends[mousemenu]) {
                                   drawoptsettings(selopt);
                                   domenu=selopt+1;
                              }
                              break;
                         case CONTROLMENU:
                              if (selopt < menustarts[domenu]+2) {
                                   _setcolor(8);
                                   _rectangle(_GFILLINTERIOR,24,100,vc.numxpixels-24,
                                        vc.numypixels-24);
                                   _setcolor(15);
                                   _rectangle(_GBORDER,24,100,vc.numxpixels-24,
                                        vc.numypixels-24);
                                   gprintf(-1,104,15,"CURRENT SETTINGS");
                                   _setcolor(0);
                                   _moveto(25,116);
                                   _lineto(vc.numxpixels-25,116);
                                   if (selopt == menustarts[domenu]) {
                                        domenu=MOUSEMENU;
                                   }
                                   else {
                                        domenu=JSTICKMENU;
                                   }
                                   mousemenu=domenu;
                                   drawoptsettings(domenu-1);
                                   oldselopt=selopt=menustarts[domenu];
                                   menuopthilite(selopt,1);
                              }
                              else {
                                   docontrolaction(domenu,selopt);
                              }
                              break;
                         case SOUNDMENU:
                              switch (selopt) {
                              case SOUNDMENUSTART:
                                   _setcolor(8);
                                   _rectangle(_GFILLINTERIOR,234,136,415,157);
                                   if (musicon) {
                                        menu[selopt].label=musicoffstr;
                                        musicon=0;
                                   }
                                   else {
                                        menu[selopt].label=musiconstr;
                                        musicon=1;
                                   }
                                   option[8]=musicon;
                                   showsoundsettings();
                                   drawmenuopt(selopt,selopt);
                                   break;
                              case SOUNDMENUSTART+1:
                                   _setcolor(8);
                                   _rectangle(_GFILLINTERIOR,234,152,415,172);
                                   if (soundon) {
                                        menu[selopt].label=soundoffstr;
                                        soundon=0;
                                   }
                                   else {
                                        menu[selopt].label=soundonstr;
                                        soundon=1;
                                   }
                                   option[10]=soundon;
                                   showsoundsettings();
                                   drawmenuopt(selopt,selopt);
                                   break;
                              case SOUNDMENUSTART+2:
                                   runaudioexe(mousemenu);
                                   break;
                              case SOUNDMENUSTART+3:
                                   menuopthilite(selopt,0);
                                   selopt=domenu-1;
                                   menuopthilite(selopt,1);
                                   oldselopt=selopt;
                                   domenu=0;
                                   break;
                              }
                              break;
                         case VIDEOMENU:
                              switch (selopt) {
                              case VIDEOMENUSTART:
                                   option[0]=1;
                                   option[6]=0x60;
                                   break;
                              case VIDEOMENUSTART+1:
                                   option[0]=1;
                                   option[6]=0x62;
                                   break;
                              }
                              showvideosettings();
                              break;
                         case EXITMENU:
                              switch (selopt) {
                              case EXITMENUSTART:
                                   if (eg_writeprefs()) {
                                        done=1;
                                        atexit( func1 );
                                   }
                                   else {
                                        done=2;
                                   }
                                   break;
                              case EXITMENUSTART+1:
                                   done=1;
                                   break;
                              default:
                                   menuopthilite(selopt,0);
                                   selopt=domenu-1;
                                   menuopthilite(selopt,1);
                                   oldselopt=selopt;
                                   domenu=0;
                                   break;
                              }
                              break;
                         case MOUSEMENU:
                              switch (selopt) {
                              case MOUSEMENUSTART:
                                   option[3]^=1;
                                   _setcolor(menu[selopt].back);
                                   _rectangle(_GFILLINTERIOR,216,menu[selopt].y,
                                        216+48,menu[selopt].y+FONTHEIGHT);
                                   gprintf(menu[selopt].x,menu[selopt].y,
                                        menu[selopt].fore,menu[selopt].label);
                                   gprintf(216,menu[selopt].y,14,
                                        option[3] == 0 ? "NO" : "YES");
                                   break;
                              case MOUSEMENUSTART+1:
                              case MOUSEMENUSTART+2:
                              case MOUSEMENUSTART+3:
                                   butopt=NUMOPTS+NUMKEYS;
                                   if (selopt == MOUSEMENUSTART+2) {
                                        butopt++;
                                   }
                                   else if (selopt == MOUSEMENUSTART+3) {
                                       butopt += 7;
                                   }
                                   menuopthilite(selopt,0);
                                   savdomenu=domenu;
                                   domenu=ACTIONMENU;
                                   savselopt=selopt;
                                   oldselopt=selopt=menustarts[domenu];
                                   drawoptsettings(domenu-1);
                                   menuopthilite(selopt,1);
                                   break;
                            // EG Always Mlook
                            case MOUSEMENUSTART+4:
                                eg_mlooklock^=1;
                                _setcolor(menu[selopt].back);
                                _rectangle(_GFILLINTERIOR,216,menu[selopt].y,
                                    216+48,menu[selopt].y+FONTHEIGHT);
                                gprintf(menu[selopt].x,menu[selopt].y,
                                    menu[selopt].fore,menu[selopt].label);
                                gprintf(216,menu[selopt].y,14,
                                    eg_mlooklock == 0 ? "NO" : "YES");
                                break;
                            // EG Invert Mlook
                            case MOUSEMENUSTART+5:
                                eg_mlookinvert^=1;
                                _setcolor(menu[selopt].back);
                                _rectangle(_GFILLINTERIOR,216,menu[selopt].y,
                                216+48,menu[selopt].y+FONTHEIGHT);
                                gprintf(menu[selopt].x,menu[selopt].y,
                                menu[selopt].fore,menu[selopt].label);
                                gprintf(216,menu[selopt].y,14,
                                eg_mlookinvert == 0 ? "NO" : "YES");
                                break;
                            // EG Always run
                            case MOUSEMENUSTART+6:
                                 runlock^=1;
                                 _setcolor(menu[selopt].back);
                                 _rectangle(_GFILLINTERIOR,216,menu[selopt].y,
                                      216+48,menu[selopt].y+FONTHEIGHT);
                                 gprintf(menu[selopt].x,menu[selopt].y,
                                      menu[selopt].fore,menu[selopt].label);
                                 gprintf(216,menu[selopt].y,14,
                                      runlock == 0 ? "NO" : "YES");
                                 break;                                     
                            case MOUSEMENUSTART+7:
                                 domenu=CONTROLMENU;
                                 oldselopt=selopt=menustarts[domenu];
                                 drawoptsettings(domenu-1);
                                 menuopthilite(selopt,1);
                                 break;
                            }
                              break;
                         case JSTICKMENU:
                              switch (selopt) {
                              case JSTICKMENUSTART:
                                   option[NUMOPTS+NUMKEYS+2]^=1;
                                   _setcolor(menu[selopt].back);
                                   _rectangle(_GFILLINTERIOR,216,menu[selopt].y,
                                        216+48,menu[selopt].y+FONTHEIGHT);
                                   gprintf(menu[selopt].x,menu[selopt].y,
                                        menu[selopt].fore,menu[selopt].label);
                                   gprintf(216,menu[selopt].y,14,
                                        option[NUMOPTS+NUMKEYS+2] == 0 ? "NO" : "YES");
                                   break;
                              case JSTICKMENUSTART+1:
                              case JSTICKMENUSTART+2:
                              case JSTICKMENUSTART+3:
                              case JSTICKMENUSTART+4:
                                   butopt=(NUMOPTS+NUMKEYS)+(selopt-(JSTICKMENUSTART+1))+3;
                                   menuopthilite(selopt,0);
                                   savdomenu=domenu;
                                   domenu=ACTIONMENU;
                                   savselopt=selopt;
                                   oldselopt=selopt=menustarts[domenu];
                                   drawoptsettings(domenu-1);
                                   menuopthilite(selopt,1);
                                   break;
                              case JSTICKMENUSTART+5:
                                   domenu=CONTROLMENU;
                                   oldselopt=selopt=menustarts[domenu];
                                   drawoptsettings(domenu-1);
                                   menuopthilite(selopt,1);
                                   break;
                              }
                              break;
                         case ACTIONMENU:
                              i=(selopt-ACTIONMENUSTART)+4;
                              option[butopt]=i;
                              _setcolor(8);
                              _rectangle(_GFILLINTERIOR,vc.numxpixels>>1,100,
                                   vc.numxpixels-24,vc.numypixels-24);
                              domenu=savdomenu;
                              oldselopt=selopt=savselopt;
                              drawoptsettings(domenu-1);
                              menuopthilite(selopt,1);
                              break;
                         }
                    }
                    for (i=menustarts[0] ; i < menuends[0] ; i++) {
                         if (menu[i].label == NULL) {
                              continue;
                         }
                         len=strlen(menu[i].label)*FONTWIDTH;
                         if ((x=menu[i].x) == -1) {
                              x=centerx(i);
                         }
                         if (mx > x && mx < x+len) {
                              if (my > menu[i].y && my < menu[i].y+FONTHEIGHT) {
                                   if (i != selopt) {
                                        selopt=i;
                                        drawmenuopt(oldselopt,selopt);
                                        oldselopt=selopt;
                                        mousemenu=0;
                                   }
                              }
                         }
                    }
                    if (domenu != 0) {
                         for (i=menustarts[domenu] ; i < menuends[domenu] ; i++) {
                              if (menu[i].label == NULL) {
                                   continue;
                              }
                              len=strlen(menu[i].label)*FONTWIDTH;
                              if ((x=menu[i].x) == -1) {
                                   x=centerx(i);
                              }
                              if (mx > x && mx < x+len) {
                                   if (my > menu[i].y && my < menu[i].y+FONTHEIGHT) {
                                        if (i != selopt) {
                                             selopt=i;
                                             drawmenuopt(oldselopt,selopt);
                                             oldselopt=selopt;
                                             mousemenu=domenu;
                                        }
                                   }
                              }
                         }
                    }
                    showmouse();
               }
          }
     } while (!done);
     if (mouseon) {
          hidemouse();
     }
     setBIOSvmode(omode);
     if (crashstr != NULL) {
          printf("\nERROR: %s\n",crashstr);
          getch();
          exit(1);
     }

     //printf("\nType WHAVEN to run the game.\n");

     exit(0);
}

void func1(void) {

     printf("\nUse EGWH1 for Witchaven or EGWH2 for Witchaven II\n");

}


void eg_parse_config(void)
{
    FILE *input_file;
    char buffer[1024];
    char name[80];
    char number[8];
    int i, j, cap, done, value;
    
    done = i = j = 0;

    input_file = fopen("egpref.cfg", "r");
    if(input_file)
    {
        while(!done)
        {
            while(1)
            {
                if(!fgets(buffer, 1024, input_file))
                {
                    done = 1;
                    break;
                }
                /* Eat leading whitespace */
                i = 0;
                while(buffer[i] == ' ' || buffer[i] == '\t')
                {
                    ++i;
                }

                /* Loop until we get a line that's relevant                              */
                /* Once it's not a comment or blank line, we can break out.              */
                if (buffer[i] != '#' && buffer[i] != '\n' && buffer[i] != '\0')
                    break;
                
            }

            cap = i + 79;
            j = 0;

            while(buffer[i] != ' ' && buffer[i] != '\t' && buffer[i] != '\n' && buffer[i] != '\0' && i < cap)
            {
                name[j] = buffer[i];
                ++i;
                ++j;
            }
            name[j] = '\0';
    
            /* Eat in-between whitespace */
            while(buffer[i] == ' ' || buffer[i] == '\t')
            {
                ++i;
            }

            cap = i + 7;
            j = 0;

            while(buffer[i] != '\n' && buffer[i] != '\0' && i < cap)
            {
                number[j] = buffer[i];
                ++i;
                ++j;
            }

            number[j] = '\0';
            
            value = atoi(number);
            // Debug
            //printf("read value: %d\n", value);

            /* Now slot the value into anything appropriate */
            /* User prefs */
            if(!stricmp(name, "video_mode"))
                if(value == 1)
                    option[6] = 0x62;
                else
                    option[6] = 0x60;
            if(!stricmp(name, "key_up"))
                keyopt[KEYFWD] = value;
            if(!stricmp(name, "key_down"))
                keyopt[KEYBACK] = value;
            if(!stricmp(name, "key_left"))
                keyopt[KEYLEFT] = value;
            if(!stricmp(name, "key_right"))
                keyopt[KEYRIGHT] = value;
            if(!stricmp(name, "key_speed"))
                keyopt[KEYRUN] = value;
            if(!stricmp(name, "key_strafe"))
                keyopt[KEYSTRAFE] = value;
            if(!stricmp(name, "key_fire"))
                keyopt[KEYFIRE] = value;
            if(!stricmp(name, "key_use"))
                keyopt[KEYUSE] = value;
            if(!stricmp(name, "key_jump"))
                keyopt[KEYJUMP] = value;
            if(!stricmp(name, "key_crouch"))
                keyopt[KEYCROUCH] = value;
            if(!stricmp(name, "key_lookup"))
                keyopt[KEYLKUP] = value;
            if(!stricmp(name, "key_lookdown"))
                keyopt[KEYLKDN] = value;
            if(!stricmp(name, "key_lookcenter"))
                keyopt[KEYCNTR] = value;
            if(!stricmp(name, "key_strafeleft"))
                keyopt[KEYSTFL] = value;
            if(!stricmp(name, "key_straferight"))
                keyopt[KEYSTFR] = value;
            if(!stricmp(name, "key_map"))
                keyopt[KEYMAP] = value;
            if(!stricmp(name, "key_zoomin"))
                keyopt[KEYZOOMI] = value;
            if(!stricmp(name, "key_zoomout"))
                keyopt[KEYZOOMO] = value;
            if(!stricmp(name, "key_usepotion"))
                keyopt[KEYUSEP] = value;
            if(!stricmp(name, "key_lookmode"))
                keyopt[KEYLOOKING] = value;
            if(!stricmp(name, "key_cast"))
                keyopt[KEYCAST] = value;
            if(!stricmp(name, "key_flyup"))
                keyopt[KEYFLYUP] = value;
            if(!stricmp(name, "key_flydown"))
                keyopt[KEYFLYDN] = value;
            if(!stricmp(name, "key_flystop"))
                keyopt[KEYUNCAST] = value;
            if(!stricmp(name, "key_nextpotion"))
                keyopt[KEYCYCPOTIONUP] = value;
            if(!stricmp(name, "key_prevpotion"))
                keyopt[KEYCYCPOTIONDN] = value;
            if(!stricmp(name, "key_shield"))
                keyopt[KEYSHIELD] = value;
            if(!stricmp(name, "mouse_b1action"))
                option[NUMOPTS+NUMKEYS] = value;
            if(!stricmp(name, "mouse_b2action"))
                option[NUMOPTS+NUMKEYS+1] = value;
            if(!stricmp(name, "mouse_b3action"))
                option[NUMOPTS+NUMKEYS+7] = value;
            if(!stricmp(name, "joystick_enable"))
                option[NUMOPTS+NUMKEYS+2] = value;
            if(!stricmp(name, "joystick_b1action"))
                option[NUMOPTS+NUMKEYS+3] = value;
            if(!stricmp(name, "joystick_b2action"))
                option[NUMOPTS+NUMKEYS+4] = value;
            if(!stricmp(name, "joystick_b3action"))
                option[NUMOPTS+NUMKEYS+5] = value;
            if(!stricmp(name, "joystick_b4action"))
                option[NUMOPTS+NUMKEYS+6] = value;				
            if(!stricmp(name, "mouse_x_sens"))
                mousxspeed = value;
            if(!stricmp(name, "mouse_y_sens"))
                mousyspeed = value;
            if(!stricmp(name, "gamma"))
                brightness = value;
            if(!stricmp(name, "vol_sound"))
                digilevel = value;
            if(!stricmp(name, "vol_music"))
                musiclevel = value;
            if(!stricmp(name, "gore"))
                goreon = value;
            if(!stricmp(name, "difficulty"))
                difficulty = value;
            if(!stricmp(name, "mlooklock"))
                eg_mlooklock = value;
            if(!stricmp(name, "mlookinvert"))
                eg_mlookinvert = value;
            if(!stricmp(name, "runlock"))
                runlock = value;            
            
            // Loop until we get marked done in the reading bit
        }
        fclose(input_file); /* Make sure to close it */
        // FIXME: TEMP HACK
        for(i = 0; i < NUMKEYS; i++)
            option[i+NUMOPTS] = keyopt[i];
        /* Protect against out-of-bounds values */
        /* User prefs */
        //if (mousxspeed < 0) mousxspeed = 0;
        //if (mousxspeed > 16) mousxspeed = 16;
        //if (mousyspeed < 0) mousyspeed = 0;
        //if (mousyspeed > 16) mousyspeed = 16;
        //if (brightness < 0) brightness = gbrightness = 0;
        //if (brightness > 8) brightness = gbrightness = 8;
        //if (goreon != 0) goreon = 1; /* EG: could potentially make an EXTREME GORE version later? */
        //if (difficulty < 1) difficulty = 1;
        //if (difficulty > 4) difficulty = 4;
        if (eg_mlooklock > 1) eg_mlooklock = 1;
        if (eg_mlooklock < 0) eg_mlooklock = 0;
        if (eg_mlookinvert > 1) eg_mlookinvert = 1;
        if (eg_mlookinvert < 0) eg_mlookinvert = 0;
		if (option[NUMOPTS+NUMKEYS+2] > 1) option[NUMOPTS+NUMKEYS+2] = 1;
    }
}

int eg_writeprefs(void)
{
    int i;
    FILE *fp;
    
    // EG NOTE - OUTPUT HAPPENS HERE
    // EG FIXME: TEMP HACK
     for(i = 0; i < NUMKEYS; i++)
         keyopt[i] = option[i+NUMOPTS];
     //here
     // EG 22 Aug 2017: Write in new format
     fp=fopen("egpref.cfg","w");
     if (fp != NULL) {
        fprintf(fp, "## EGwhaven config for Witchaven II\n");
          fprintf(fp, "\n## GRAPHICS\n");
          if(option[6] == 0x62)
            fprintf(fp, "video_mode 1\n");
          else
            fprintf(fp, "video_mode 2\n");
        fprintf(fp, "gamma %d\n", brightness);
        fprintf(fp, "\n## CONTROLS\n");
        fprintf(fp, "key_up %d\n", keyopt[KEYFWD]);
        fprintf(fp, "key_down %d\n", keyopt[KEYBACK]);
        fprintf(fp, "key_left %d\n", keyopt[KEYLEFT]);
        fprintf(fp, "key_right %d\n", keyopt[KEYRIGHT]);
        fprintf(fp, "key_speed %d\n", keyopt[KEYRUN]);
        fprintf(fp, "key_strafe %d\n", keyopt[KEYSTRAFE]);
        fprintf(fp, "key_fire %d\n", keyopt[KEYFIRE]);
        fprintf(fp, "key_use %d\n", keyopt[KEYUSE]);
        fprintf(fp, "key_jump %d\n", keyopt[KEYJUMP]);
        fprintf(fp, "key_crouch %d\n", keyopt[KEYCROUCH]);
        fprintf(fp, "key_lookup %d\n", keyopt[KEYLKUP]);
        fprintf(fp, "key_lookdown %d\n", keyopt[KEYLKDN]);
        fprintf(fp, "key_lookcenter %d\n", keyopt[KEYCNTR]);
        fprintf(fp, "key_strafeleft %d\n", keyopt[KEYSTFL]);
        fprintf(fp, "key_straferight %d\n", keyopt[KEYSTFR]);
        fprintf(fp, "key_map %d\n", keyopt[KEYMAP]);
        fprintf(fp, "key_zoomin %d\n", keyopt[KEYZOOMI]);
        fprintf(fp, "key_zoomout %d\n", keyopt[KEYZOOMO]);
        fprintf(fp, "key_usepotion %d\n", keyopt[KEYUSEP]);
        fprintf(fp, "key_lookmode %d\n", keyopt[KEYLOOKING]);
        fprintf(fp, "key_cast %d\n", keyopt[KEYCAST]);
        // TODO? intervening keys here
        fprintf(fp, "key_flyup %d\n", keyopt[KEYFLYUP]);
        fprintf(fp, "key_flydown %d\n", keyopt[KEYFLYDN]);
		fprintf(fp, "key_flystop %d\n", keyopt[KEYUNCAST]);
		fprintf(fp, "key_nextpotion %d\n", keyopt[KEYCYCPOTIONUP]);
		fprintf(fp, "key_prevpotion %d\n", keyopt[KEYCYCPOTIONDN]);
        fprintf(fp, "key_shield %d\n", keyopt[KEYSHIELD]);
        
        fprintf(fp, "mouse_x_sens %d\n", mousxspeed);
        fprintf(fp, "mouse_y_sens %d\n", mousyspeed);
        fprintf(fp, "mouse_b1action %d\n", option[NUMOPTS+NUMKEYS]);
        fprintf(fp, "mouse_b2action %d\n", option[NUMOPTS+NUMKEYS+1]);
        fprintf(fp, "mouse_b3action %d\n", option[NUMOPTS+NUMKEYS+7]);
        fprintf(fp, "mlooklock %d\n", eg_mlooklock);
        fprintf(fp, "mlookinvert %d\n", eg_mlookinvert);
		// 16 Oct 2017 - Joystick
		fprintf(fp, "joystick_enable %d\n", option[NUMOPTS+NUMKEYS+2]);
		fprintf(fp, "joystick_b1action %d\n", option[NUMOPTS+NUMKEYS+3]);
		fprintf(fp, "joystick_b2action %d\n", option[NUMOPTS+NUMKEYS+4]);
		fprintf(fp, "joystick_b3action %d\n", option[NUMOPTS+NUMKEYS+5]);
		fprintf(fp, "joystick_b4action %d\n", option[NUMOPTS+NUMKEYS+6]);
        fprintf(fp, "\n## SOUND\n");
        fprintf(fp, "vol_sound %d\n", digilevel);
        fprintf(fp, "vol_music %d\n", musiclevel);
        fprintf(fp, "\n## GAME PREFS\n");
        fprintf(fp, "gore %d\n", goreon);
        fprintf(fp, "difficulty %d\n", difficulty);
        fprintf(fp, "runlock %d\n", runlock);
          
        fclose(fp);
        return 1;
     }
     else
         return 0;
}